"""
    { Operations } for GraphQL
"""
import fastberry as fb

from . import forms, manager, types

@fb.gql
class Demo:
    """GraphQL Manager"""

    class Meta:
        """Class Meta-Data"""

        app = False
        model = "Category"

    class Query:
        """GraphQL Query"""

        async def detail(item: fb.item) -> fb.query(types.Category):
            """(Detail-Operation) Read The Docs"""

            return types.Category(_id=1, id="$encoded_id$", name="Cool")

        async def search(
            search: forms.Search, pagination: fb.pagination
        ) -> fb.edges(types.Category):
            """(Search-Operation) Read The Docs"""

            # Do Something if input { IS VALID } ...
            if search.input.is_valid:
                print("\n# Client's Inputs #")
                print(search.input.data)
                print(pagination.input.data)

            # Get Data from the { Database } ...
            results = await manager.Category.all()
            print("\n# Database Response #")
            print(results)

            # Return { Page }
            return fb.page(
                edges=[
                    types.Category(_id=1, id=1, name="Computer"),
                    types.Category(_id=2, id=2, name="Radio"),
                ],
                length=2,
                pages=1,
            )

    class Mutation:
        """GraphQL Mutation"""

        async def create(form: forms.Category) -> fb.mutation(types.Category):
            """(Create-Operation) Read The Docs"""

            # Client's Input
            if form.input.is_valid:
                print(form.input.data)

            # Errors
            errors_messages = []

            # IF error . . .
            errors_messages.append(fb.error(type="input", text="Error for Demo!"))
            return fb.errors(messages=errors_messages)



"""
    { Operations } for GraphQL
"""
import fastberry as fb

# Import Once's per project in any App.
# IF you want the built-in (Forms) to be register in the GraphQL API.
from fastberry import item, pagination

form = fb.input("form")

@form
class Category:
    """(Form) Read The Docs"""

    name = fb.value(
        str,
        required=True,
    )

@form(name="SearchEngine")
class Search:
    """(Form) Read The Docs"""

    name = fb.value(
        str,  # Single Typing
        default=None,
    )
    count = fb.value(
        int,
        default=None,
    )    
    cash = fb.value(
        float,
        default=None,
    )
    amount = fb.value(
        fb.decimal,
        default=None,
    )
    start_date = fb.value(
        fb.date,
        default=None,
    )
    end_datetime = fb.value(
        fb.datetime,
        default=None,
    )
    timestamp = fb.value(
        fb.time,
        default=None,
        required=False,
    )



"""
    Strawberry Pagination
"""

import dataclasses as dc
import typing

import fastberry as fb
import strawberry

DOC = """
**Setting**: `(all: true)` 

**Returns**: **All** of the **items** in the **Database** (Mostly For **Admin(s) Use Only**).
"""


@strawberry.input(description=DOC)
@dc.dataclass
class Pagination:
    """GraphQL Pagination"""

    page: int = 1
    limit: typing.Optional[int] = 50
    sort_by: typing.Optional[str] = "-id"
    all: typing.Optional[bool] = False

    def __post_init__(self):
        """Add Search Parameters to Query"""
        controller = fb.App()
        page = controller.pagination(
            page=self.page,
            limit=self.limit,
        )
        self.page = page.page
        self.limit = page.limit



"""
    Commands
"""
import click

from .cmd import cli

TITLE = "Fastberry"

HELP_TEXT = f"""
Welcome to { TITLE }
"""


def commands(commands: list):
    """Collect (Click) Commands"""
    command_sources = [cli]
    for active in commands:
        if isinstance(active.object, click.core.Group):
            command_sources.append(active.object)
    return click.CommandCollection(name=TITLE, help=HELP_TEXT, sources=command_sources)



# -*- coding: utf-8 -*-
""" [Middleware]
    Get { Authorization } from the { Cookies } and inject it to the { Headers }.
"""

from fastberry import BaseMiddleware


class AuthenticatedCookieMiddleware(BaseMiddleware):
    """Get Authorization From Cookie"""

    async def dispatch(self, request, call_next):
        """Process Request and Inject Header"""

        if (
            "Authorization" not in request.headers
            and "Authorization" in request.cookies
        ):
            access_token = request.cookies.get("Authorization")
            request.headers.__dict__["_list"].append(
                (
                    "authorization".encode(),
                    f"Bearer {access_token}".encode(),
                )
            )
        response = await call_next(request)
        return response



# -*- coding: utf-8 -*-
"""
    API - GraphQL
"""

# Fastberry
import fastberry as fb

# Create your API (GraphQL) here.


@fb.gql
class Demo:
    """Demo Api"""

    class Meta:
        """Meta-Data"""

        app = False
        model = "user"

    class Query:
        """Query"""

        async def search(info, pagination: fb.pagination) -> str:
            """Read the Docs"""
            print(info)
            print(pagination.input)
            return "Search"

        async def detail(info, item: fb.item) -> str:
            """Read the Docs"""
            print(info)
            print(item.input)
            return "Detail"

    class Mutation:
        """Mutation"""

        async def create(info) -> str:
            """Read the Docs"""
            print(info)
            return "Create"

        async def update(info) -> str:
            """Read the Docs"""
            print(info)
            return "Update"

        async def delete(info) -> str:
            """Read the Docs"""
            print(info)
            return "Delete"
